import numpy as np
from math import *
#from numba import jit
#import matplotlib
#matplotlib.use('GTKAgg')
import pylab as pl
#import pylab as pl

from sklearn.preprocessing import normalize
from functools import partial

import os
import glob
try:
    import multiprocessing as mp
    parallel=True
except:
    print("Multiprocessing module cannot be loaded!")
    parallel=False;

#from OwnFunctions import *

class AgentData(object):
    ''' A class containing all the data structures for agents ''' 
    def __init__(self,paraSystem):
        N=paraSystem.N;

        # Cartesian coordinates
        self.pos=np.zeros((N,2));
        self.vel=np.zeros((N,2));
        self.uv =np.zeros((N,2));
        self.up =np.zeros((N,2));
        
        # social force variables
        self.force=np.zeros((N,2))
        self.force_att=np.zeros((N,2))
        self.force_rep=np.zeros((N,2))
        self.force_alg=np.zeros((N,2))

        self.neighbors_rep=np.zeros((N,1))
        self.neighbors_alg=np.zeros((N,1))
        self.neighbors_att=np.zeros((N,1))

        # agent speeds
        self.speed=np.ones((N,1));
        # agent direction of motion (polar angle)
        self.phi=np.zeros(N);

        # social force strengths
        self.repstrength=np.ones(N);
        self.attstrength=np.ones(N);
        self.algstrength=np.ones(N);
        #social force ranges
        self.reprange=np.ones(N);
        self.attrange=np.ones(N);
        self.algrange=np.ones(N);
        #social force ranges
        self.repsteepness=np.ones(N);
        self.attsteepness=np.ones(N);
        self.algsteepness=np.ones(N);
        #individual noise strengths
        self.sigmap=np.zeros(N)
    
    # helper methods for resetting/normalizing/updating
    def ResetForces(self):
        self.force.fill(0.0)
        self.force_att.fill(0.0)
        self.force_rep.fill(0.0)
        self.force_alg.fill(0.0)
        self.neighbors_rep.fill(0.0)
        self.neighbors_alg.fill(0.0)
        self.neighbors_att.fill(0.0)

    def NormalizeForces(self):
        self.force_att=normalize(self.force_att,axis=1,norm='l2')
        self.force_rep=normalize(self.force_rep,axis=1,norm='l2')
        self.force_alg=normalize(self.force_alg,axis=1,norm='l2')

    def UpdateDirection(self):
        self.phi=np.arctan2(self.vel[:,1],self.vel[:,0])

    def UpdateCartesian(self):
        self.uv[:,0] =np.cos(self.phi);
        self.uv[:,1] =np.sin(self.phi);
        self.up=np.copy(self.uv[:,::-1])
        self.up[:,0]*=-1
        self.vel=np.multiply(np.reshape(self.speed,(-1,1)),self.uv)

class BaseParams(object):
    ''' A class defining global model parameters ''' 
    def __init__(self,time=100,dt=0.05,output=1.0,L=50,N=10,IC=0,BC=0,int_type='matrix'):
        self.time=time;     # simulation time
        self.dt=dt;         # time step for integration of SDEs
        self.steps=int(self.time/self.dt)   # number of simulation steps
        self.output=output;                 # time intervals between output
        self.stepout=int(self.output/self.dt)  # step intervals between outputs 
        self.outsteps=int(self.steps/self.stepout) # number out outputs
        self.L=L        # linear size of the arena
        self.N=N        # number of agents
        self.IC=IC;     # switch for initial conditions
        self.BC=BC;     # switch for boundary conditions
        self.int_type=int_type # switch for interaction calculation type


class AgentParams(object):
    ''' Agent Parameter class - setting the parameters
        Required input: paraSystem
    '''
    def __init__(self,paraSystem,
                 speed0=1.0,
                 wallrepdist=1,wallrepstrength=5.0,
                 attstrength=0.2,attrange=100,attsteepness=-20,
                 repstrength=1.0,reprange=1.0,repsteepness=-20,
                 algstrength=0.5,algrange=4,algsteepness=-20,
                 blindangle=0.0*np.pi,blindanglesteepness=50,
                 kneigh=6,
                 noisep=0.01
                 ):
        
        self.speed0=speed0; # global speed for all agents 
        
        self.sigmap=np.sqrt(2.0*paraSystem.dt*noisep);

        self.wallrepdist=wallrepdist;
        self.wallrepstrength=wallrepstrength;
        
        self.attstrength=attstrength;
        self.attrange=attrange;
        self.attsteepness=attsteepness;

        self.algstrength=algstrength;
        self.algrange=algrange;
        self.algsteepness=algsteepness;

        self.repstrength=repstrength;
        self.reprange=reprange;   
        self.repsteepness=repsteepness;
        
        self.blindangle=blindangle;
        self.cosblindangle=np.cos(np.pi-0.5*self.blindangle)
        self.blindanglesteepness=blindanglesteepness;

        self.kneigh=kneigh

##################### BASIC RUN FUNCTION ##################################################
def SingleRunGlobal(paraSystem,paraAgents,agentData=None):
    ''' perform a single simulation run '''

    ### INITIALIZATION ##################################
    # initialize agents if not provided at run time 
    if(agentData==None):
        agentData=AgentData(paraSystem)
        InitAgents(agentData,paraSystem,paraAgents) 
    # initialize output arrays
    outdata=SaveOutData(0,agentData,None,paraSystem)

    ### PERFORM TIME INTEGRATION ######################
    for s in range(1,paraSystem.steps+1):
        # reset social forces
        agentData.ResetForces()  # call to method of the agentData class
        # calculate new social foces
        CalcInteractionGlobal(agentData,paraSystem,paraAgents)
        UpdateTotalSocialForce(agentData,paraSystem,paraAgents)
        # update direction of motion, positions/velocity vectors
        UpdateCoordinates( agentData,paraSystem,paraAgents)
        
        ### OUTPUT ###################
        if((s % paraSystem.stepout)==0):
            #print('t=%02f' % (para.dt*s))
            outdata=SaveOutData(s,agentData,outdata,paraSystem)

    return outdata 

def InitAgents(agentData,paraSystem,paraAgents):
    N=paraSystem.N

    if(paraSystem.IC==10):
        agentData.pos=0.25*paraSystem.L*np.random.random((N,2))
    else:
        agentData.pos=paraSystem.L*np.random.random((N,2))
    # set polar coordinates
    agentData.phi  =2.*np.pi*np.random.random(N)
    agentData.speed=paraAgents.speed0*np.ones(N)
    #update Cartesian coordinates
    agentData.UpdateCartesian()

    #initialize noise
    agentData.sigmap=np.ones(N)*paraAgents.sigmap
   
    # set interaction parameters
    agentData.repstrength*=paraAgents.repstrength;
    agentData.algstrength*=paraAgents.algstrength;
    agentData.attstrength*=paraAgents.attstrength;

    agentData.reprange*=paraAgents.reprange;
    agentData.algrange*=paraAgents.algrange;
    agentData.attrange*=paraAgents.attrange;

    agentData.repsteepness*=paraAgents.repsteepness;
    agentData.algsteepness*=paraAgents.algsteepness;
    agentData.attsteepness*=paraAgents.attsteepness;

    return

def SaveOutData(s,agentData,outdata,paraSystem):
    
    if(outdata==None):
        outdata=dict()
        outdata['pos']=[]
        outdata['vel']=[]
        outdata['phi']=[]
        outdata['speed']=[]
        outdata['force']=[]
        outdata['timer_s']=[]
        outdata['t']=[]

    outdata['pos'].append(np.copy(agentData.pos))
    outdata['vel'].append(np.copy(agentData.vel))
    outdata['phi'].append(np.copy(agentData.phi))
    outdata['speed' ].append(np.copy(agentData.speed))
    outdata['force' ].append(np.copy(agentData.force))
    outdata['t'].append(s*paraSystem.dt)

    return outdata

def CalcInteractionGlobal(agentData,paraSystem,paraAgents):
    ''' calculate the interaction by simple N^2 loop over all pairs'''
    for i in range(paraSystem.N):
        for j in range(i+1,paraSystem.N):
            CalcInteractionPair(i,j,agentData,paraSystem,paraAgents)
    return

def UpdateTotalSocialForce(agentData,paraSystem,paraAgents):

    if(paraSystem.int_type=='voronoi'):
        NormalizeByCount(agentData)
    else:
        agentData.NormalizeForces()

    agentData.force=( agentData.repstrength*agentData.force_rep.T
                     +agentData.attstrength*agentData.force_att.T
                     +agentData.algstrength*agentData.force_alg.T).T
    
    return

def UpdateCoordinates(agentData,paraSystem,paraAgents):
    ''' update of coordinates for all agents ''' 
    #print(agentData.force[0,0:2])
    dphi=CalculateDirectionChange(agentData,paraSystem,paraAgents)
    #calculate new angle
    agentData.phi+=dphi
    # ensure angle between (0,2pi)
    agentData.phi=np.fmod(agentData.phi,2.0*np.pi)
    #update Cartesian velocity vectors and positions
    agentData.UpdateCartesian()
    agentData.pos+=agentData.vel*paraSystem.dt
    if(paraSystem.BC==0):
        agentData.pos=CalcBoundaryPeriodic(agentData.pos,paraSystem.L)
    if(paraSystem.BC==1):
        agentData=CalcBoundaryReflecting(agentData,paraSystem)

    return 

###############################################################################################################################################
def CalculateDirectionChange(agentData,paraSystem,paraAgents):
    #project forces on polar angle
    forcep=np.sum(agentData.force*agentData.up,axis=1)
    #deterministic increments
    dphi  = forcep*paraSystem.dt
    #add direction noise
    if(paraAgents.sigmap>0.0):
        noiseP=np.random.normal(0.0,agentData.sigmap,size=paraSystem.N)
        dphi  +=noiseP

    #account for inertia 
    dphi/=(agentData.speed+1e-10)

    return dphi

def PeriodicDist(x,y,L=10.0,dim=2):
    ''' Returns the distance vector of two position vectors x,y
        by tanking periodic boundary conditions into account.

        Input parameters: L - system size, dim - no. of dimension
    '''
    distvec=(y-x)
    distvec_periodic=np.copy(distvec)
    distvec_periodic[distvec<-0.5*L]+=L
    distvec_periodic[distvec>0.5*L]-=L

    return distvec_periodic

def SigThresh(x,x0=0.5,steepness=10):
    ''' Sigmoid function f(x)=1/2*(tanh(a*(x-x0)+1)
        
        Input parameters:
        -----------------
        x:  function argument
        x0: position of the transition point (f(x0)=1/2)
        steepness:  parameter setting the steepness of the transition.
                    (positive values: transition from 0 to 1, negative values: 
                    transition from 1 to 0)
    '''
    return 0.5*(np.tanh(steepness*(x-x0))+1)

def CalcDistVecMatrix(pos,L,BC):
    X=np.reshape(pos[:,0],(-1,1))
    Y=np.reshape(pos[:,1],(-1,1))
    dX=np.subtract(X,X.T)
    dY=np.subtract(Y,Y.T)
    if(BC==0):
        dX[dX>+0.5*L]-=L;
        dY[dY>+0.5*L]-=L;
        dX[dX<-0.5*L]+=L;
        dY[dY<-0.5*L]+=L;
    distmatrix=np.sqrt(dX**2+dY**2)
    return distmatrix,dX,dY

def CalcVelDiffVecMatrix(vel):
    VX=np.reshape(vel[:,0],(-1,1))
    VY=np.reshape(vel[:,1],(-1,1))
    dVX=np.subtract(VX,VX.T)
    dVY=np.subtract(VY,VY.T)
    return dVX,dVY

def CalcForceVecMatrix(factormatrix,distmatrix,dX,dY,normalize=True):
    if(normalize):
        dUX=np.divide(dX,distmatrix)
        dUY=np.divide(dY,distmatrix)
        FX=np.multiply(factormatrix,dUX)
        FY=np.multiply(factormatrix,dUY)
    else:
        FX=np.multiply(factormatrix,dX)
        FY=np.multiply(factormatrix,dY)

    return FX,FY

def CalcAlgVecMatrix(factormatrix,dUX,dUY):
    FX=-np.multiply(factormatrix,dUX)
    FY=-np.multiply(factormatrix,dUY)
    return FX,FY

def CalcFactorMatrix(distmatrix,force_range,force_steepness):
    #factormatrix=np.int32(distmatrix<force_range)
    
    factormatrix=SigThresh(distmatrix,force_range,force_steepness)
    #factormatrix=np.multiply(force_strength,tmpmatrix)
    return factormatrix

def CalcDistanceVec(pos_i,pos_j,L,BC,dim=2):
    ''' Convenience function to calculate distance'''
    if(BC==0):
        distvec=PeriodicDist(pos_i,pos_j,L,dim)
    else:
        distvec=pos_j-pos_i

    return distvec

def PlainInit():
    paraSystem=BaseParams()
    paraAgents=AgentParams(paraSystem)
    return paraSystem,paraAgents

def UniqueEdgesFromTriangulation(tri):
    e1=tri.simplices[:,0:2]
    e2=tri.simplices[:,1:3]
    e3=tri.simplices[:,::2]
    edges=np.vstack((e1,e2,e3))
    edges.sort(axis=0)
    edges_c = np.ascontiguousarray(edges).view(np.dtype((np.void, edges.dtype.itemsize * edges.shape[1])))
    _, idx = np.unique(edges_c, return_index=True)
    edges_unique = edges[idx]
    return edges_unique

def CalcInteractionMatrix(agentData,paraSystem,paraAgents):
    distmatrix,dX,dY=CalcDistVecMatrix(agentData.pos,paraSystem.L,paraSystem.BC)
    dVX,dVY=CalcVelDiffVecMatrix(agentData.vel)
    # repulsion
    factor_M=CalcFactorMatrix(distmatrix,agentData.reprange,agentData.repsteepness)
    FX,FY=CalcForceVecMatrix(factor_M,distmatrix,dX,dY)
    agentData.force_rep[:,0]=np.nanmean(FX,axis=1)
    agentData.force_rep[:,1]=np.nanmean(FY,axis=1)
    # alignment
    factor_M=CalcFactorMatrix(distmatrix,agentData.algrange,agentData.algsteepness)
    FX,FY=CalcAlgVecMatrix(factor_M,dVX,dVY)
    agentData.force_alg[:,0]=np.nanmean(FX,axis=1)
    agentData.force_alg[:,1]=np.nanmean(FY,axis=1)
    # attraction
    factor_M=CalcFactorMatrix(distmatrix,agentData.attrange,agentData.attsteepness)
    FX,FY=CalcForceVecMatrix(factor_M,distmatrix,dX,dY,normalize=False)
    agentData.force_att[:,0]=-np.nanmean(FX,axis=1)
    agentData.force_att[:,1]=-np.nanmean(FY,axis=1)

    return distmatrix 

def CalcInteractionVoronoi(agentData,paraSystem,paraAgents):
    from scipy.spatial import Delaunay
    tri=Delaunay(agentData.pos)
    edges=UniqueEdgesFromTriangulation(tri)
    for e in edges:
        if(e[0]!=e[1]):
            CalcInteractionPair(e[0],e[1],agentData,paraSystem,paraAgents)
    return edges


def CalcSingleRepForce(i,distvec,dist,agentData):
    repfactor=SigThresh(dist,agentData.reprange[i],agentData.repsteepness[i]);
    agentData.force_rep[i]-=repfactor*distvec;
    agentData.neighbors_rep[i]+=1;
    return

def CalcSingleAlgForce(i,dvel,dist,agentData):
    algfactor=SigThresh(dist,agentData.algrange[i],agentData.algsteepness[i]);
    agentData.force_alg[i]+=algfactor*dvel;
    agentData.neighbors_alg[i]+=1;
    agentData.force_alg[i,:]=0;
    return

def CalcSingleAttForce(i,distvec,dist,agentData):
    attfactor=SigThresh(dist,agentData.attrange[i],agentData.attsteepness[i]);
    agentData.force_att[i]+=attfactor*distvec;
    agentData.neighbors_att[i]+=1;
    return

def CalcInteractionPair(i,j,agentData,paraSystem,paraAgents):
    L  =paraSystem.L
    BC =paraSystem.BC
    # calculate distance vector and scalar   
    distvec=CalcDistanceVec(agentData.pos[i],agentData.pos[j],L,BC)
    dist=0.0
    for d in range(2):
        dist+=distvec[d]**2
    dist=np.sqrt(dist);

    #### calculate repulsion ###
    if(agentData.repstrength[i]>0.0):
        CalcSingleRepForce(i,distvec,dist,agentData)
        CalcSingleRepForce(j,-distvec,dist,agentData)
    ############################    
    
    ### calculate alignment ###
    if(agentData.algstrength[i]>0.0):
        dvel=agentData.vel[j]-agentData.vel[i]
        CalcSingleAlgForce(i,+dvel,dist,agentData)
        CalcSingleAlgForce(j,-dvel,dist,agentData)
    ############################    

    ### calculate attraction ###
    if(agentData.attstrength[i]>0.0):
        CalcSingleAttForce(i,distvec,dist,agentData)
        CalcSingleAttForce(j,-distvec,dist,agentData)
    ############################    
    return

def NormalizeByCount(agentData):

    #repulsion
    idx_non_zero_count=np.nonzero(agentData.neighbors_rep)[0]
    agentData.force_rep[idx_non_zero_count]/=agentData.neighbors_rep[idx_non_zero_count]
    #alignment
    idx_non_zero_count=np.nonzero(agentData.neighbors_alg)[0]
    agentData.force_alg[idx_non_zero_count]/=agentData.neighbors_alg[idx_non_zero_count]
    #alignment
    idx_non_zero_count=np.nonzero(agentData.neighbors_att)[0]
    agentData.force_att[idx_non_zero_count]/=agentData.neighbors_att[idx_non_zero_count]

    return

def CalcBoundaryPeriodic(pos,L):

    posx=pos[:,0]
    posy=pos[:,1]

    posx[posx>L]=posx[posx>L]-L 
    posy[posy>L]=posy[posy>L]-L 
    posx[posx<0.0]=posx[posx<0.0]+L 
    posy[posy<0.0]=posy[posy<0.0]+L 

    pos[:,0]=posx
    pos[:,1]=posy

    return pos

def CalcBoundaryReflecting(agentData,paraSystem):

    posx=agentData.pos[:,0]
    posy=agentData.pos[:,1]

    velx=agentData.vel[:,0]
    vely=agentData.vel[:,1]

    velx[posx>paraSystem.L]*=-1.0; 
    vely[posy>paraSystem.L]*=-1.0; 
    velx[posx<0]*=-1.0; 
    vely[posy<0]*=-1.0; 

    posx[posx>paraSystem.L]=2*paraSystem.L-posx[posx>paraSystem.L] 
    posy[posy>paraSystem.L]=2*paraSystem.L-posy[posy>paraSystem.L] 
    posx[posx<0.0]=-posx[posx<0.0] 
    posy[posy<0.0]=-posy[posy<0.0] 


    agentData.pos[:,0]=posx
    agentData.pos[:,1]=posy
    agentData.vel[:,0]=velx
    agentData.vel[:,1]=vely

    return agentData

def CalcPolarization(phi):
    ux=np.cos(phi)
    uy=np.sin(phi)
    mean_ux=np.mean(ux)
    mean_uy=np.mean(uy)
    return np.sqrt(mean_ux**2+mean_uy**2)


def SingleRun(paraSystem,paraAgents,agentData=None):
    print("Performing a single run.")

    if(agentData==None):
        agentData=AgentData(paraSystem)
        InitAgents(agentData,paraSystem,paraAgents) 

    outdata=SaveOutData(0,agentData,None,paraSystem)

    for s in range(1,paraSystem.steps+1):
        agentData.ResetForces()
        
        if(paraSystem.int_type=='global'):
            CalcInteractionGlobal(agentData,paraSystem,paraAgents)
        elif(paraSystem.int_type=='matrix'):
            distmatrix=CalcInteractionMatrix(agentData,paraSystem,paraAgents)
            edges=None;
        elif(paraSystem.int_type=='voronoi'):
            edges=CalcInteractionVoronoi(agentData,paraSystem,paraAgents)
            distmatrix=None;
        else:
            CalcInteractionKDTree(agentData,paraSystem,paraAgents)
            edges=None;

        UpdateTotalSocialForce(agentData,paraSystem,paraAgents)
        UpdateCoordinates( agentData,paraSystem,paraAgents)
        
        if((s % paraSystem.stepout)==0):
            #print('t=%02f' % (para.dt*s))
            outdata=SaveOutData(s,agentData,outdata,paraSystem)
    print('Done!')
    return outdata 


def SingleRunVoronoi(paraSystem,paraAgents,agentData=None):
    if(agentData==None):
        agentData=AgentData(paraSystem)
        InitAgents(agentData,paraSystem,paraAgents) 

    outdata=SaveOutData(0,agentData,None,paraSystem)

    for s in range(1,paraSystem.steps+1):
        agentData.ResetForces()
        edges=CalcInteractionVoronoi(agentData,paraSystem,paraAgents)
        UpdateTotalSocialForce(agentData,paraSystem,paraAgents)
        UpdateCoordinates( agentData,paraSystem,paraAgents)
        
        if((s % paraSystem.stepout)==0):
            outdata=SaveOutData(s,agentData,outdata,paraSystem)
    #print('Done!')
    return outdata


def MakeMovie(outdata,paraSystem,paraAgents,parallel=parallel):
    if(parallel):

        no_of_threads=mp.cpu_count();
        print("Generating frames for movie using {} threads".format(no_of_threads))
        pool=mp.Pool(no_of_threads)
        gen_frame=partial(PlotFrame,outdata,paraSystem,paraAgents)
        time_array=np.arange(paraSystem.outsteps)
        #print(time_array)
        pool.map(gen_frame,time_array)
        pool.close()
    else:
        for s in range(paraSystem.outsteps):
            PlotFrame(outdata,paraSystem,paraAgents,s)

    enc_codec='mpeg4'
    moviefile='animation.mp4'
    ffmpeg_command='ffmpeg -framerate 30 -i f%04d.png -c:v '+enc_codec+' -pix_fmt yuv420p '+moviefile
    print(ffmpeg_command)
    try:
        os.system(ffmpeg_command)
    except:
        print("Encoding video failed. Is ffmpeg installed correctly?")

    return

def PlotFrame(outdata,paraSystem,paraAgents,s):
    
    tail_length=5
    pl.figure(figsize=(8,8))
    pl.subplot(aspect='equal')
    pl.xlim(0, paraSystem.L)
    pl.ylim(0, paraSystem.L)
    x=outdata['pos'][s][:,0]
    y=outdata['pos'][s][:,1]

    pl.plot(x, y, 'ro')
    if(s>tail_length):
        postail=np.array(outdata['pos'][s-tail_length:s])
        xtail=np.reshape(postail[:,:,0],(-1,1))
        ytail=np.reshape(postail[:,:,1],(-1,1))
        pl.plot(xtail, ytail, 'r.')

    pl.title('t={}'.format(paraSystem.output*s))
    #pl.tight_layout()
    pl.savefig('f%04d.png' % s,dpi=300)
    #pl.clf()


    pl.close()

    return


def RunAnimate(paraSystem,paraAgents,doblit=False):
    agentData=AgentData(paraSystem)
    InitAgents(agentData,paraSystem,paraAgents) 
    fig=pl.figure(99,figsize=(10,10))
    ax = pl.subplot()
    ax.set_aspect('equal')
    ax.set_xlim(0, paraSystem.L)
    ax.set_ylim(0, paraSystem.L)
    ax.hold(True)

    x=agentData.pos[:,0]
    y=agentData.pos[:,1]
    pl.show(False)
    pl.draw()
    points = ax.plot(x, y, 'ro')[0]
    pointstail = ax.plot(x, y, 'r.',alpha=0.2)[0]

    if doblit:
        # cache the background
        background = fig.canvas.copy_from_bbox(ax.bbox)

    outdata=SaveOutData(0,agentData,None,paraSystem)
    #if(paraSystem.int_type=='global'):
    #    edges=[]
    #    for i in range(paraSystem.N):
    #        for j in range(i,paraSystem.N):
    #            edges.append([i,j])

    for s in range(1,paraSystem.steps+1):
        agentData.ResetForces()
        if(paraSystem.int_type=='global'):
            CalcInteractionGlobal(agentData,paraSystem,paraAgents)
        elif(paraSystem.int_type=='matrix'):
            distmatrix=CalcInteractionMatrix(agentData,paraSystem,paraAgents)
        elif(paraSystem.int_type=='voronoi'):
            edges=CalcInteractionVoronoi(agentData,paraSystem,paraAgents)
            distmatrix=None;

        UpdateTotalSocialForce(agentData,paraSystem,paraAgents)
        UpdateCoordinates( agentData,paraSystem,paraAgents)
        
        if((s % paraSystem.stepout)==0):
            print('t=%02f' % (paraSystem.dt*s))
            
            outdata=SaveOutData(s,agentData,outdata,paraSystem)
            x=agentData.pos[:,0]
            y=agentData.pos[:,1]
            if(paraSystem.output<1.0):
                tail_length=int(3./(paraSystem.output))
            else:
                tail_length=5
            postail=np.array(outdata['pos'][-tail_length:-1])
            xtail=np.reshape(postail[:,:,0],(-1,1))
            ytail=np.reshape(postail[:,:,1],(-1,1))

            points.set_data(x, y)
            pointstail.set_data(xtail,ytail)

            if(paraSystem.BC==-1):
                meanx=np.mean(x)
                meany=np.mean(y)
                ax.set_xlim(meanx-0.5*paraSystem.L,meanx+0.5*paraSystem.L)
                ax.set_ylim(meany-0.5*paraSystem.L,meany+0.5*paraSystem.L)

            if doblit:
                # restore background
                fig.canvas.restore_region(background)

                # redraw just the points
                ax.draw_artist(points)

                # fill in the axes rectangle
                fig.canvas.blit(ax.bbox)

            else:
                # redraw everything
                ax.set_title('t=%02f' % (paraSystem.dt*s))
                fig.canvas.draw()

    #raw_input("Press Enter to continue...")
    pl.close(fig)

    return outdata,agentData 

    



